@@ -20,17 +20,14 @@ from .types import (
     COMPONENT_CONSTRAINTS,
     AllNeXusComponents,
     Beamline,
-    CalibratedBeamline,
-    CalibratedDetector,
-    CalibratedMonitor,
     Component,
     DetectorBankSizes,
-    DetectorData,
     DetectorPositionOffset,
+    EmptyDetector,
+    EmptyMonitor,
     Filename,
     GravityVector,
     Measurement,
-    MonitorData,
     MonitorPositionOffset,
     MonitorType,
     NeXusAllComponentLocationSpec,
@@ -46,8 +43,9 @@ from .types import (
     Position,
     PreopenNeXusFile,
     RawChoppers,
+    RawDetector,
+    RawMonitor,
     RunType,
-    SampleRun,
     TimeInterval,
     UniqueComponent,
 )
@@ -361,13 +359,11 @@ def get_calibrated_detector(
     # all.
     offset: DetectorPositionOffset[RunType],
     bank_sizes: DetectorBankSizes,
-) -> CalibratedDetector[RunType]:
+) -> EmptyDetector[RunType]:
     """
     Extract the data array corresponding to a detector's signal field.
 
-    The returned data array includes coords and masks pertaining directly to the
-    signal values array, but not additional information about the detector. The
-    data array is reshaped to the logical detector shape, which by folding the data
+    The data array is reshaped to the logical detector shape, by folding the data
     array along the detector_number dimension.
 
     Parameters
@@ -389,57 +385,26 @@ def get_calibrated_detector(
     # If the NXdetector in the file is not 1-D, we want to match the order of dims.
     # zip_pixel_offsets otherwise yields a vector with dimensions in the order given
     # by the x/y/z offsets.
-    offsets = snx.zip_pixel_offsets(da.coords).transpose(da.dims).copy()
+    offsets = snx.zip_pixel_offsets(da.coords)
+    # Get the dims in the order of the detector data array, but filter out dims that
+    # don't exist in the offsets (e.g. the detector data may have a 'time' dimension).
+    dims = [dim for dim in da.dims if dim in offsets.dims]
+    offsets = offsets.transpose(dims).copy()
     # We use the unit of the offsets as this is likely what the user expects.
     if transform.value.unit is not None and transform.value.unit != '':
         transform_value = transform.value.to(unit=offsets.unit)
     else:
         transform_value = transform.value
     position = transform_value * offsets
-    return CalibratedDetector[RunType](
+    return EmptyDetector[RunType](
         da.assign_coords(position=position + offset.to(unit=position.unit))
     )
 
 
-def assemble_beamline(
-    detector: CalibratedDetector[RunType],
-    source_position: Position[snx.NXsource, RunType],
-    sample_position: Position[snx.NXsample, RunType],
-    gravity: GravityVector,
-) -> CalibratedBeamline[RunType]:
-    """
-    Add beamline information (gravity vector, source- and sample-position) to detector.
-
-    This is performed separately and after :py:func:`get_calibrated_detector` to avoid
-    as false dependency of, e.g., the reshaped detector numbers on the sample position.
-    The latter can change during a run, e.g., for a rotating sample. The detector
-    numbers might be used, e.g., to mask certain detector pixels, and should not depend
-    on the sample position.
-
-    Parameters
-    ----------
-    detector:
-        NeXus detector group.
-    source_position:
-        Position of the neutron source.
-    sample_position:
-        Position of the sample.
-    gravity:
-        Gravity vector.
-    """
-    return CalibratedBeamline[RunType](
-        detector.assign_coords(
-            source_position=source_position,
-            sample_position=sample_position,
-            gravity=gravity,
-        )
-    )
-
-
 def assemble_detector_data(
-    detector: CalibratedBeamline[RunType],
-    event_data: NeXusData[snx.NXdetector, RunType],
-) -> DetectorData[RunType]:
+    detector: EmptyDetector[RunType],
+    neutron_data: NeXusData[snx.NXdetector, RunType],
+) -> RawDetector[RunType]:
     """
     Assemble a detector data array with event data.
 
@@ -449,14 +414,15 @@ def assemble_detector_data(
     ----------
     detector:
         Calibrated detector data array.
-    event_data:
-        Event data array.
+    neutron_data:
+        Neutron data array (events or histogram).
     """
-    grouped = nexus.group_event_data(
-        event_data=event_data, detector_number=detector.coords['detector_number']
-    )
-    return DetectorData[RunType](
-        _add_variances(grouped)
+    if neutron_data.bins is not None:
+        neutron_data = nexus.group_event_data(
+            event_data=neutron_data, detector_number=detector.coords['detector_number']
+        )
+    return RawDetector[RunType](
+        _add_variances(neutron_data)
         .assign_coords(detector.coords)
         .assign_masks(detector.masks)
     )
@@ -466,7 +432,7 @@ def get_calibrated_monitor(
     monitor: NeXusComponent[MonitorType, RunType],
     offset: MonitorPositionOffset[RunType, MonitorType],
     source_position: Position[snx.NXsource, RunType],
-) -> CalibratedMonitor[RunType, MonitorType]:
+) -> EmptyMonitor[RunType, MonitorType]:
     """
     Extract the data array corresponding to a monitor's signal field.
 
@@ -483,7 +449,7 @@ def get_calibrated_monitor(
         Position of the neutron source.
     """
     monitor = nexus.compute_component_position(monitor)
-    return CalibratedMonitor[RunType, MonitorType](
+    return EmptyMonitor[RunType, MonitorType](
         nexus.extract_signal_data_array(monitor).assign_coords(
             position=monitor['position'] + offset.to(unit=monitor['position'].unit),
             source_position=source_position,
@@ -492,9 +458,9 @@ def get_calibrated_monitor(
 
 
 def assemble_monitor_data(
-    monitor: CalibratedMonitor[RunType, MonitorType],
+    monitor: EmptyMonitor[RunType, MonitorType],
     data: NeXusData[MonitorType, RunType],
-) -> MonitorData[RunType, MonitorType]:
+) -> RawMonitor[RunType, MonitorType]:
     """
     Assemble a monitor data array with event data.
 
@@ -508,7 +474,7 @@ def assemble_monitor_data(
         Data array with neutron counts.
     """
     da = data.assign_coords(monitor.coords).assign_masks(monitor.masks)
-    return MonitorData[RunType, MonitorType](_add_variances(da))
+    return RawMonitor[RunType, MonitorType](_add_variances(da))
 
 
 def parse_disk_choppers(
@@ -543,6 +509,19 @@ def _drop(
     }
 
 
+class _EmptyField:
+    """Empty field that can replace a missing detector_number in NXdetector."""
+
+    def __init__(self, sizes: dict[str, int]):
+        self.attrs = {}
+        self.sizes = sizes.copy()
+        self.dims = tuple(sizes.keys())
+        self.shape = tuple(sizes.values())
+
+    def __getitem__(self, key: Any) -> sc.Variable:
+        return sc.zeros(dims=self.dims, shape=self.shape, unit=None, dtype='int32')
+
+
 class _StrippedDetector(snx.NXdetector):
     """Detector definition without large geometry or event data for ScippNexus.
 
@@ -552,8 +531,36 @@ class _StrippedDetector(snx.NXdetector):
     def __init__(
         self, attrs: dict[str, Any], children: dict[str, snx.Field | snx.Group]
     ):
-        children = _drop(children, (snx.NXoff_geometry, snx.NXevent_data))
-        children['data'] = children['detector_number']
+        if 'detector_number' in children:
+            data = children['detector_number']
+        else:
+            # We get the 'data' sizes before the NXdata is dropped
+            if 'data' not in children:
+                raise KeyError(
+                    "StrippedDetector: Cannot determine shape of the detector. "
+                    "No 'detector_number' was found, and the 'data' entry is missing."
+                )
+            if 'value' not in children['data']:
+                raise KeyError(
+                    "StrippedDetector: Cannot determine shape of the detector. "
+                    "The 'data' entry has no 'value'."
+                )
+            # We drop any time-related dimension from the data sizes, as they are not
+            # relevant for the detector geometry/shape.
+            data = _EmptyField(
+                sizes={
+                    dim: size
+                    for dim, size in children['data']['value'].sizes.items()
+                    if dim not in ('time', 'frame_time')
+                }
+            )
+
+        children = _drop(
+            children, (snx.NXoff_geometry, snx.NXevent_data, snx.NXdata, snx.NXlog)
+        )
+
+        children['data'] = data
+
         super().__init__(attrs=attrs, children=children)
 
 
@@ -567,7 +574,7 @@ class _DummyField:
         self.shape = (0,)
 
     def __getitem__(self, key: Any) -> sc.Variable:
-        return sc.empty(dims=self.dims, shape=self.shape, unit=None)
+        return sc.zeros(dims=self.dims, shape=self.shape, unit=None, dtype='int32')
 
 
 class _StrippedMonitor(snx.NXmonitor):
@@ -606,14 +613,16 @@ def _assign_values_as_variances(var: sc.Variable) -> sc.Variable:
     return var
 
 
-def load_beamline_metadata_from_nexus(file_spec: NeXusFileSpec[SampleRun]) -> Beamline:
+def load_beamline_metadata_from_nexus(
+    file_spec: NeXusFileSpec[RunType],
+) -> Beamline[RunType]:
     """Load beamline metadata from a sample NeXus file."""
     return nexus.load_metadata(file_spec.value, Beamline)
 
 
 def load_measurement_metadata_from_nexus(
-    file_spec: NeXusFileSpec[SampleRun],
-) -> Measurement:
+    file_spec: NeXusFileSpec[RunType],
+) -> Measurement[RunType]:
     """Load measurement metadata from a sample NeXus file."""
     return nexus.load_metadata(file_spec.value, Measurement)
 
@@ -654,7 +663,6 @@ _detector_providers = (
     no_detector_position_offset,
     load_nexus_sample,
     get_calibrated_detector,
-    assemble_beamline,
     assemble_detector_data,
 )
 
@@ -683,16 +691,12 @@ def LoadMonitorWorkflow(
 
 
 def LoadDetectorWorkflow(
-    *,
-    run_types: Iterable[sciline.typing.Key],
-    monitor_types: Iterable[sciline.typing.Key],
+    *, run_types: Iterable[sciline.typing.Key]
 ) -> sciline.Pipeline:
     """Generic workflow for loading detector data from a NeXus file."""
     wf = sciline.Pipeline(
         (*_common_providers, *_detector_providers),
-        constraints=_gather_constraints(
-            run_types=run_types, monitor_types=monitor_types
-        ),
+        constraints=_gather_constraints(run_types=run_types, monitor_types=[]),
     )
     wf[DetectorBankSizes] = DetectorBankSizes({})
     wf[PreopenNeXusFile] = PreopenNeXusFile(False)
