@@ -24,25 +24,24 @@ from .lut import (
 )
 from .types import (
     DetectorLtotal,
-    DetectorTofData,
     MonitorLtotal,
-    MonitorTofData,
     PulseStrideOffset,
     TimeOfFlightLookupTable,
     TimeOfFlightLookupTableFilename,
+    ToaDetector,
+    TofDetector,
+    TofMonitor,
 )
 from .workflow import GenericTofWorkflow
 
 __all__ = [
     "DetectorLtotal",
-    "DetectorTofData",
     "DiskChoppers",
     "DistanceResolution",
     "GenericTofWorkflow",
     "LookupTableRelativeErrorThreshold",
     "LtotalRange",
     "MonitorLtotal",
-    "MonitorTofData",
     "NumberOfSimulatedNeutrons",
     "PulsePeriod",
     "PulseStride",
@@ -53,7 +52,10 @@ __all__ = [
     "TimeOfFlightLookupTable",
     "TimeOfFlightLookupTableFilename",
     "TimeResolution",
+    "ToaDetector",
+    "TofDetector",
     "TofLookupTableWorkflow",
+    "TofMonitor",
     "providers",
     "simulate_chopper_cascade_using_tof",
 ]
