@@ -4,28 +4,32 @@ from datetime import UTC, datetime
 from pathlib import Path
 
 import pytest
+import sciline as sl
 import scipp as sc
 import scippnexus as snx
 from scipp.testing import assert_identical
 
-from ess.reduce.nexus import compute_component_position, workflow
+from ess.reduce.nexus import compute_component_position, load_from_path, workflow
 from ess.reduce.nexus.types import (
     BackgroundRun,
     Beamline,
-    DetectorData,
     EmptyBeamRun,
+    EmptyDetector,
     Filename,
     FrameMonitor0,
     FrameMonitor1,
     FrameMonitor2,
     Measurement,
-    MonitorData,
     MonitorType,
     NeXusComponentLocationSpec,
+    NeXusFileSpec,
+    NeXusLocationSpec,
     NeXusName,
     NeXusTransformation,
     PreopenNeXusFile,
     RawChoppers,
+    RawDetector,
+    RawMonitor,
     RunType,
     SampleRun,
     TimeInterval,
@@ -333,9 +337,9 @@ def test_get_calibrated_detector_forwards_masks(
 
 
 @pytest.fixture
-def calibrated_detector() -> workflow.CalibratedDetector[SampleRun]:
+def calibrated_detector() -> workflow.EmptyDetector[SampleRun]:
     detector_number = sc.arange('detector_number', 6, unit=None)
-    return workflow.CalibratedDetector[SampleRun](
+    return workflow.EmptyDetector[SampleRun](
         sc.DataArray(
             sc.empty_like(detector_number),
             coords={
@@ -446,8 +450,8 @@ def test_get_calibrated_monitor_subtracts_offset_from_position(
 
 
 @pytest.fixture
-def calibrated_monitor() -> workflow.CalibratedMonitor[SampleRun, FrameMonitor1]:
-    return workflow.CalibratedMonitor[SampleRun, FrameMonitor1](
+def calibrated_monitor() -> workflow.EmptyMonitor[SampleRun, FrameMonitor1]:
+    return workflow.EmptyMonitor[SampleRun, FrameMonitor1](
         sc.DataArray(
             sc.scalar(0),
             coords={'position': sc.vector([1.0, 2.0, 3.0], unit='m')},
@@ -553,7 +557,7 @@ def test_load_event_monitor_workflow(loki_tutorial_sample_run_60250: Path) -> No
     wf = LoadMonitorWorkflow(run_types=[SampleRun], monitor_types=[FrameMonitor1])
     wf[Filename[SampleRun]] = loki_tutorial_sample_run_60250
     wf[NeXusName[FrameMonitor1]] = 'monitor_1'
-    da = wf.compute(MonitorData[SampleRun, FrameMonitor1])
+    da = wf.compute(RawMonitor[SampleRun, FrameMonitor1])
     assert 'position' in da.coords
     assert 'source_position' in da.coords
     assert da.bins is not None
@@ -565,7 +569,7 @@ def test_load_histogram_monitor_workflow(dream_coda_test_file: Path) -> None:
     wf = LoadMonitorWorkflow(run_types=[SampleRun], monitor_types=[FrameMonitor1])
     wf[Filename[SampleRun]] = dream_coda_test_file
     wf[NeXusName[FrameMonitor1]] = 'monitor_bunker'
-    da = wf.compute(MonitorData[SampleRun, FrameMonitor1])
+    da = wf.compute(RawMonitor[SampleRun, FrameMonitor1])
     assert 'position' in da.coords
     assert 'source_position' in da.coords
     assert da.bins is None
@@ -576,17 +580,40 @@ def test_load_histogram_monitor_workflow(dream_coda_test_file: Path) -> None:
 
 
 def test_load_detector_workflow(loki_tutorial_sample_run_60250: Path) -> None:
-    wf = LoadDetectorWorkflow(run_types=[SampleRun], monitor_types=[])
+    wf = LoadDetectorWorkflow(run_types=[SampleRun])
     wf[Filename[SampleRun]] = loki_tutorial_sample_run_60250
     wf[NeXusName[snx.NXdetector]] = 'larmor_detector'
-    da = wf.compute(DetectorData[SampleRun])
+    da = wf.compute(RawDetector[SampleRun])
     assert 'position' in da.coords
-    assert 'sample_position' in da.coords
-    assert 'source_position' in da.coords
     assert da.bins is not None
     assert da.dims == ('detector_number',)
 
 
+def test_load_histogram_detector_workflow(tbl_commissioning_orca_file: Path) -> None:
+    wf = LoadDetectorWorkflow(run_types=[SampleRun])
+    wf[Filename[SampleRun]] = tbl_commissioning_orca_file
+    wf[NeXusName[snx.NXdetector]] = 'orca_detector'
+    da = wf.compute(RawDetector[SampleRun])
+    assert 'position' in da.coords
+    assert da.bins is None
+    assert 'time' in da.dims
+    assert da.ndim == 3
+
+
+def test_load_empty_histogram_detector_workflow(
+    tbl_commissioning_orca_file: Path,
+) -> None:
+    wf = LoadDetectorWorkflow(run_types=[SampleRun])
+    wf[Filename[SampleRun]] = tbl_commissioning_orca_file
+    wf[NeXusName[snx.NXdetector]] = 'orca_detector'
+    da = wf.compute(EmptyDetector[SampleRun])
+    assert 'position' in da.coords
+    assert da.bins is None
+    # The empty detector has no time dimension, only the dimensions of the geometry
+    assert 'time' not in da.dims
+    assert da.ndim == 2
+
+
 @pytest.mark.parametrize('preopen', [True, False])
 def test_generic_nexus_workflow(
     preopen: bool, loki_tutorial_sample_run_60250: Path
@@ -596,13 +623,11 @@ def test_generic_nexus_workflow(
     wf[NeXusName[FrameMonitor1]] = 'monitor_1'
     wf[NeXusName[snx.NXdetector]] = 'larmor_detector'
     wf[PreopenNeXusFile] = preopen
-    da = wf.compute(DetectorData[SampleRun])
+    da = wf.compute(RawDetector[SampleRun])
     assert 'position' in da.coords
-    assert 'sample_position' in da.coords
-    assert 'source_position' in da.coords
     assert da.bins is not None
     assert da.dims == ('detector_number',)
-    da = wf.compute(MonitorData[SampleRun, FrameMonitor1])
+    da = wf.compute(RawMonitor[SampleRun, FrameMonitor1])
     assert 'position' in da.coords
     assert 'source_position' in da.coords
     assert da.bins is not None
@@ -633,7 +658,7 @@ def test_generic_nexus_workflow_load_beamline_metadata(
 ) -> None:
     wf = GenericNeXusWorkflow(run_types=[SampleRun], monitor_types=[])
     wf[Filename[SampleRun]] = bifrost_simulated_elastic
-    beamline = wf.compute(Beamline)
+    beamline = wf.compute(Beamline[SampleRun])
 
     assert beamline.name == 'BIFROST'
     assert beamline.facility == 'ESS'
@@ -646,7 +671,7 @@ def test_generic_nexus_workflow_load_measurement_metadata(
     wf = GenericNeXusWorkflow(run_types=[SampleRun], monitor_types=[])
     wf[Filename[SampleRun]] = loki_tutorial_sample_run_60250
     wf[Filename[BackgroundRun]] = loki_tutorial_background_run_60248
-    measurement = wf.compute(Measurement)
+    measurement = wf.compute(Measurement[SampleRun])
 
     assert measurement.title == 'My experiment'
     assert measurement.experiment_id == 'p1234'
@@ -663,14 +688,14 @@ def test_generic_nexus_workflow_includes_only_given_run_and_monitor_types() -> N
     graph = wf.underlying_graph
 
     # Check some examples to avoid relying entirely on complicated loops below.
-    assert DetectorData[SampleRun] in graph
-    assert DetectorData[BackgroundRun] not in graph
-    assert MonitorData[SampleRun, FrameMonitor1] in graph
-    assert MonitorData[SampleRun, FrameMonitor2] not in graph
-    assert MonitorData[SampleRun, FrameMonitor0] in graph
-    assert MonitorData[BackgroundRun, FrameMonitor0] not in graph
-    assert MonitorData[BackgroundRun, FrameMonitor1] not in graph
-    assert MonitorData[BackgroundRun, FrameMonitor2] not in graph
+    assert RawDetector[SampleRun] in graph
+    assert RawDetector[BackgroundRun] not in graph
+    assert RawMonitor[SampleRun, FrameMonitor1] in graph
+    assert RawMonitor[SampleRun, FrameMonitor2] not in graph
+    assert RawMonitor[SampleRun, FrameMonitor0] in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor0] not in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor1] not in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor2] not in graph
     assert RawChoppers[SampleRun] in graph
     assert RawChoppers[BackgroundRun] not in graph
 
@@ -705,14 +730,14 @@ def test_generic_nexus_workflow_includes_only_given_run_types() -> None:
     graph = wf.underlying_graph
 
     # Check some examples to avoid relying entirely on complicated loops below.
-    assert DetectorData[EmptyBeamRun] in graph
-    assert DetectorData[SampleRun] not in graph
-    assert MonitorData[EmptyBeamRun, FrameMonitor1] in graph
-    assert MonitorData[EmptyBeamRun, FrameMonitor2] in graph
-    assert MonitorData[EmptyBeamRun, FrameMonitor0] in graph
-    assert MonitorData[SampleRun, FrameMonitor1] not in graph
-    assert MonitorData[SampleRun, FrameMonitor2] not in graph
-    assert MonitorData[SampleRun, FrameMonitor0] not in graph
+    assert RawDetector[EmptyBeamRun] in graph
+    assert RawDetector[SampleRun] not in graph
+    assert RawMonitor[EmptyBeamRun, FrameMonitor1] in graph
+    assert RawMonitor[EmptyBeamRun, FrameMonitor2] in graph
+    assert RawMonitor[EmptyBeamRun, FrameMonitor0] in graph
+    assert RawMonitor[SampleRun, FrameMonitor1] not in graph
+    assert RawMonitor[SampleRun, FrameMonitor2] not in graph
+    assert RawMonitor[SampleRun, FrameMonitor0] not in graph
     assert RawChoppers[EmptyBeamRun] in graph
     assert RawChoppers[SampleRun] not in graph
 
@@ -729,16 +754,16 @@ def test_generic_nexus_workflow_includes_only_given_monitor_types() -> None:
     graph = wf.underlying_graph
 
     # Check some examples to avoid relying entirely on complicated loops below.
-    assert DetectorData[SampleRun] in graph
-    assert DetectorData[BackgroundRun] in graph
-    assert MonitorData[SampleRun, TransmissionMonitor] in graph
-    assert MonitorData[SampleRun, FrameMonitor1] in graph
-    assert MonitorData[SampleRun, FrameMonitor2] not in graph
-    assert MonitorData[SampleRun, FrameMonitor0] not in graph
-    assert MonitorData[BackgroundRun, TransmissionMonitor] in graph
-    assert MonitorData[BackgroundRun, FrameMonitor1] in graph
-    assert MonitorData[BackgroundRun, FrameMonitor2] not in graph
-    assert MonitorData[BackgroundRun, FrameMonitor0] not in graph
+    assert RawDetector[SampleRun] in graph
+    assert RawDetector[BackgroundRun] in graph
+    assert RawMonitor[SampleRun, TransmissionMonitor] in graph
+    assert RawMonitor[SampleRun, FrameMonitor1] in graph
+    assert RawMonitor[SampleRun, FrameMonitor2] not in graph
+    assert RawMonitor[SampleRun, FrameMonitor0] not in graph
+    assert RawMonitor[BackgroundRun, TransmissionMonitor] in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor1] in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor2] not in graph
+    assert RawMonitor[BackgroundRun, FrameMonitor0] not in graph
     assert RawChoppers[SampleRun] in graph
     assert RawChoppers[BackgroundRun] in graph
 
@@ -754,3 +779,48 @@ def assert_not_contains_type_arg(node: object, excluded: set[type]) -> None:
     assert not any(
         arg in excluded for arg in getattr(node, "__args__", ())
     ), f"Node {node} contains one of {excluded!r}"
+
+
+def test_generic_nexus_workflow_load_custom_field_user_affiliation(
+    loki_tutorial_sample_run_60250: Path,
+) -> None:
+    class UserAffiliation(sl.Scope[RunType, str], str):
+        """User affiliation."""
+
+    def load_user_affiliation(
+        file: NeXusFileSpec[RunType], path: NeXusName[UserAffiliation[RunType]]
+    ) -> UserAffiliation[RunType]:
+        return UserAffiliation[RunType](
+            load_from_path(NeXusLocationSpec(filename=file.value, component_name=path))
+        )
+
+    wf = GenericNeXusWorkflow(run_types=[SampleRun], monitor_types=[])
+    wf.insert(load_user_affiliation)
+    wf[Filename[SampleRun]] = loki_tutorial_sample_run_60250
+    # Path is relative to the top-level '/entry'
+    wf[NeXusName[UserAffiliation[SampleRun]]] = 'user_0/affiliation'
+    affiliation = wf.compute(UserAffiliation[SampleRun])
+    assert affiliation == 'ESS'
+
+
+def test_generic_nexus_workflow_load_custom_group_user(
+    loki_tutorial_sample_run_60250: Path,
+) -> None:
+    class UserInfo(sl.Scope[RunType, str], str):
+        """User info."""
+
+    def load_user_info(
+        file: NeXusFileSpec[RunType], path: NeXusName[UserInfo[RunType]]
+    ) -> UserInfo[RunType]:
+        return UserInfo[RunType](
+            load_from_path(NeXusLocationSpec(filename=file.value, component_name=path))
+        )
+
+    wf = GenericNeXusWorkflow(run_types=[SampleRun], monitor_types=[])
+    wf.insert(load_user_info)
+    wf[Filename[SampleRun]] = loki_tutorial_sample_run_60250
+    # Path is relative to the top-level '/entry'
+    wf[NeXusName[UserInfo]] = 'user_0'
+    user_info = wf.compute(UserInfo[SampleRun])
+    assert user_info['affiliation'] == 'ESS'
+    assert user_info['name'] == 'John Doe'
