@@ -9,7 +9,7 @@ from scippneutron.conversion.graph.beamline import beamline as beamline_graph
 from scippneutron.conversion.graph.tof import elastic as elastic_graph
 
 from ess.reduce import time_of_flight
-from ess.reduce.nexus.types import DetectorData, SampleRun
+from ess.reduce.nexus.types import AnyRun, RawDetector, SampleRun
 from ess.reduce.time_of_flight import GenericTofWorkflow, TofLookupTableWorkflow, fakes
 
 sl = pytest.importorskip("sciline")
@@ -112,7 +112,7 @@ def dream_source_position() -> sc.Variable:
 @pytest.fixture(scope="module")
 def lut_workflow_dream_choppers() -> sl.Pipeline:
     lut_wf = TofLookupTableWorkflow()
-    lut_wf[time_of_flight.DiskChoppers] = dream_choppers()
+    lut_wf[time_of_flight.DiskChoppers[AnyRun]] = dream_choppers()
     lut_wf[time_of_flight.SourcePosition] = dream_source_position()
     lut_wf[time_of_flight.NumberOfSimulatedNeutrons] = 100_000
     lut_wf[time_of_flight.SimulationSeed] = 432
@@ -130,7 +130,7 @@ def setup_workflow(
     error_threshold: float = 0.1,
 ) -> sl.Pipeline:
     pl = GenericTofWorkflow(run_types=[SampleRun], monitor_types=[])
-    pl[DetectorData[SampleRun]] = raw_data
+    pl[RawDetector[SampleRun]] = raw_data
     pl[time_of_flight.DetectorLtotal[SampleRun]] = ltotal
 
     lut_wf = lut_workflow.copy()
@@ -194,7 +194,7 @@ def test_dream_wfm(
         raw_data=raw, ltotal=ltotal, lut_workflow=lut_workflow_dream_choppers
     )
 
-    tofs = pl.compute(time_of_flight.DetectorTofData[SampleRun])
+    tofs = pl.compute(time_of_flight.TofDetector[SampleRun])
 
     # Convert to wavelength
     graph = {**beamline_graph(scatter=False), **elastic_graph("tof")}
@@ -206,14 +206,14 @@ def test_dream_wfm(
             (x.coords["wavelength"] - ref.coords["wavelength"])
             / ref.coords["wavelength"]
         )
-        assert np.nanpercentile(diff.values, 100) < 0.02
+        assert np.nanpercentile(diff.values, 99.9) < 0.02
         assert sc.isclose(ref.data.sum(), da.data.sum(), rtol=sc.scalar(1.0e-3))
 
 
 @pytest.fixture(scope="module")
 def lut_workflow_dream_choppers_time_overlap():
     lut_wf = TofLookupTableWorkflow()
-    lut_wf[time_of_flight.DiskChoppers] = dream_choppers_with_frame_overlap()
+    lut_wf[time_of_flight.DiskChoppers[AnyRun]] = dream_choppers_with_frame_overlap()
     lut_wf[time_of_flight.SourcePosition] = dream_source_position()
     lut_wf[time_of_flight.NumberOfSimulatedNeutrons] = 100_000
     lut_wf[time_of_flight.SimulationSeed] = 432
@@ -281,7 +281,7 @@ def test_dream_wfm_with_subframe_time_overlap(
         error_threshold=0.01,
     )
 
-    tofs = pl.compute(time_of_flight.DetectorTofData[SampleRun])
+    tofs = pl.compute(time_of_flight.TofDetector[SampleRun])
 
     # Convert to wavelength
     graph = {**beamline_graph(scatter=False), **elastic_graph("tof")}
@@ -292,7 +292,7 @@ def test_dream_wfm_with_subframe_time_overlap(
         sel = sc.isfinite(x.coords["wavelength"])
         y = ref.coords["wavelength"][sel]
         diff = abs((x.coords["wavelength"][sel] - y) / y)
-        assert np.nanpercentile(diff.values, 100) < 0.02
+        assert np.nanpercentile(diff.values, 99.9) < 0.02
         sum_wfm = da.hist(wavelength=100).data.sum()
         sum_ref = ref.hist(wavelength=100).data.sum()
         # Verify that we lost some neutrons that were in the overlapping region
@@ -405,7 +405,7 @@ def v20_source_position():
 @pytest.fixture(scope="module")
 def lut_workflow_v20_choppers():
     lut_wf = TofLookupTableWorkflow()
-    lut_wf[time_of_flight.DiskChoppers] = v20_choppers()
+    lut_wf[time_of_flight.DiskChoppers[AnyRun]] = v20_choppers()
     lut_wf[time_of_flight.SourcePosition] = v20_source_position()
     lut_wf[time_of_flight.NumberOfSimulatedNeutrons] = 300_000
     lut_wf[time_of_flight.SimulationSeed] = 431
@@ -464,7 +464,7 @@ def test_v20_compute_wavelengths_from_wfm(
         raw_data=raw, ltotal=ltotal, lut_workflow=lut_workflow_v20_choppers
     )
 
-    tofs = pl.compute(time_of_flight.DetectorTofData[SampleRun])
+    tofs = pl.compute(time_of_flight.TofDetector[SampleRun])
 
     # Convert to wavelength
     graph = {**beamline_graph(scatter=False), **elastic_graph("tof")}
